#include "njGamma.h"
#include "numRec.h"
#include "countTableComponent.h"
#include "gammaDistribution.h"
#include "likeDist.h"
#include "uniDistribution.h"

gammaMLDistances::gammaMLDistances(const stochasticProcess & sp,
							const VVdouble & posteriorProb,
							const MDOUBLE toll,
							const MDOUBLE maxPairwiseDistance) 
							:
  _sp(sp),_posteriorProb(posteriorProb),_toll(toll),_maxPairwiseDistance(maxPairwiseDistance) 
{}


MDOUBLE gammaMLDistances::giveDistance(	const sequence& s1,
							const sequence& s2,
							const Vdouble  * weights,
							MDOUBLE* score) const {
	const MDOUBLE ax=0,bx=1.0,cx=_maxPairwiseDistance;
	MDOUBLE dist=-1.0;
	MDOUBLE resL = -dbrent(ax,bx,cx,
		  C_eval_gammaMLDistances(_sp,s1,s2,_posteriorProb,weights),
		  C_eval_gammaMLDistances_d(_sp,s1,s2,_posteriorProb,weights),
		  _toll,
		  &dist);
	if (score) *score = resL;
	return dist;
}

// =============================
class C_evalAlphaForPairOfSeq{
private:
	const countTableComponentGam& _ctc;
	stochasticProcess& _sp;
	const MDOUBLE _branchL;
public:
	C_evalAlphaForPairOfSeq(const countTableComponentGam& ctc,
					const MDOUBLE branchL,
					stochasticProcess& sp):_ctc(ctc), _sp(sp), _branchL(branchL) {};

	MDOUBLE operator() (MDOUBLE alpha) {
		(static_cast<gammaDistribution*>(_sp.distr()))->setAlpha(alpha);
		C_evalLikeDist cev(_ctc,_sp);
		return cev(_branchL);
	};
};

// returns the best alpha.
MDOUBLE optimizeAlphaFixedDist(stochasticProcess & sp,
							   const countTableComponentGam & ctc,
							   const MDOUBLE branchL,
							   const vector<MDOUBLE>  * weights,
							   MDOUBLE* score=NULL){ // changes sp.
	MDOUBLE bestA=0;
	const MDOUBLE upperBoundOnAlpha = 15.0;
	const MDOUBLE epsilonAlphaOptimization = 0.01;
	const MDOUBLE cx=upperBoundOnAlpha;// left, midle, right limit on alpha
	const MDOUBLE bx=cx*0.3;
	const MDOUBLE ax=0.0;

	
	MDOUBLE bestQ = -brent(ax,bx,cx,
		C_evalAlphaForPairOfSeq(ctc,branchL,sp),
		epsilonAlphaOptimization,
		&bestA);
	(static_cast<gammaDistribution*>(sp.distr()))->setAlpha(bestA);
	return bestA;
}



MDOUBLE gammaMLDistances::giveInitialGuessOfDistance(
							const sequence& s1,
							const sequence& s2,
							const vector<MDOUBLE>  * weights,
							MDOUBLE* score) const {
	uniDistribution ud;
	stochasticProcess uniSp(&ud,_sp.getPijAccelerator());
	likeDist ld(uniSp);
	return (ld.giveDistance(s1,s2,weights,score));
}

MDOUBLE gammaMLDistances::giveDistanceOptAlphaForEachPairOfSequences(	const sequence& s1,
							const sequence& s2,
							const vector<MDOUBLE>  * weights,
							MDOUBLE* score) const {

	MDOUBLE toll = 0.0001;
	
	MDOUBLE resQ = 0.0;
	MDOUBLE correntDistance = giveInitialGuessOfDistance(s1,s2,weights,&resQ);
	
	countTableComponentGam ctc; // from technical reasons.
	ctc.countTableComponentAllocatePlace(_sp.alphabetSize(),_sp.categories());
	
	stochasticProcess tmpSp(_sp);
	for (int z=0; z<s1.seqLen(); ++z) {
		for (int j=0; j < tmpSp.categories(); ++j) {
            ctc.addToCounts(s1[z],s2[z],j,weights?(*weights)[z]:tmpSp.ratesProb(j));
		}
	}
	const int maxIter = 30;
	MDOUBLE resL = 0.0;
	MDOUBLE newDist = 0.0;
	for (int i=0; i < maxIter; ++i) {
		MDOUBLE bestAlpha = optimizeAlphaFixedDist(tmpSp,ctc,correntDistance,weights,score); // changes sp.
		likeDist tmpld(tmpSp); // we must create a new ld, that will include the stochastic process with the new alpha
		newDist = tmpld.giveDistance(ctc,resQ);
		if (fabs(newDist-correntDistance)<toll) break;
		correntDistance = newDist;
	}
	if (score) *score = resL;
	assert (newDist >=0);
	return newDist;
	
							}